/**  
  *  IHS_from_RGB Version 1.0, 2008-10-10, by 
  *  Martin Schlueter, i3mainz, Mainz University of Applied Sciences, Germany
  *
  *  This ImageJ plugin performs a color transformation from RGB to IHS.
  *  
  *  Based on
  *  Wolfgang GOEPFERT:
  *  Raumbezogene Informationssysteme
  *  Textbook, Wichmann, Karlsruhe, 1987.
  *
  *  To learn more please refer to the related web pages with examples and
  *  applications.
  *
  *  For consultance and practical applications concerning Mobile 3D Coordinate
  *  Measuring Techniques, 3D Digitizing, Deformation Measurement and Analysis 
  *  please contact
  *  Martin Schlueter: schlueter(at)geoinform.fh-mainz.de   
  *  
  *  Copyright (c) [2008-10-10] by Prof. Dr.-Ing. Martin Schlueter, i3mainz,
  *  Mainz University of Applied Sciences, Germany

  *  Permission is hereby granted, free of charge, to any person obtaining a 
  *  copy of this software and associated documentation files (the "Software"), 
  *  to deal in the Software without restriction, including without limitation 
  *  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
  *  and/or sell copies of the Software, and to permit persons to whom the 
  *  Software is furnished to do so, subject to the following conditions:
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
  *  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  *  DEALINGS IN THE SOFTWARE.
  *
  *  OSI Certified Open Source Software
  *
  * History
  * 2008/10/10: First version online
  *
  */

import ij.ImagePlus;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

public class IHS_from_RGB implements PlugInFilter {

    public int setup(String arg, ImagePlus imp) {
        return DOES_RGB+DOES_STACKS;
    }
    
    public void run(ImageProcessor ip) {
    
        int idiv = 3;
        int fak = 16000 * 255; // factor to keep round-off errors small
        int hdiv = fak / 255 * 3;
        int sdiv = fak / 255;
        int iadd = idiv >> 1; // iadd = idiv /2 by rightshift
        int hadd = hdiv >> 1;
        int sadd = sdiv >> 1;

        int[] pixels = (int[]) ip.getPixels();

        //iterate over all pixels
        for (int j = 0; j < pixels.length; j++) {

            //get int-packed color pixel
            int c = pixels[j];

            //extract RGB components from color pixel
            int r = (c & 0xff0000) >> 16;
            int g = (c & 0x00ff00) >> 8;
            int b = (c & 0x0000ff);
               
            //intensity
            int i = (r + g + b);
             
            //hue
            int h = 0, div = 0;
            int m = Math.min(r, Math.min(g, b));
            if (b == m) {
               div = (i - 3 * b);
               if (div == 0) 
                  h = 0;   // case r = g = b
               else
                  h = (fak * (g - b)) / div;
            }   
            else if (r == m) {
               div = (i - 3 * r);
               h = (fak * (b - r)) / div + fak;
            }
            else {
               div = (i - 3 * g);
               h = (fak * (r - g)) / div + 2 * fak;
            }
                           
            //saturation
            int s = 0;
            if (i > 0)
               s = (fak * div) / i;
            
            //re-scale i, h, s to [0-255]; add half of divisor to keep round-off errors small
            r = Math.min(255, Math.max(0, ((i + iadd) / idiv)));
            g = Math.min(255, Math.max(0, ((h + hadd) / hdiv)));
            b = Math.min(255, Math.max(0, ((s + sadd) / sdiv)));

            // reassemble color pixel
            c = ((r & 0xff)<<16) | ((g & 0xff)<<8) | b & 0xff; 
            pixels[j] = c;
        }
    }
}
